@@ -16,7 +16,7 @@
 ##' values from the MCMC chains that were inputted in
 ##' \code{\link{bf1skel}} will be used.
 ##' @param useCV Whether to use control variates for finer
-##' corrections. 
+##' corrections.
 ##' @return An array of size \code{length(linkp) * length(phi) *
 ##' length(omg) * length(kappa)} containing the Bayes factors for each
 ##' combination of the parameters.
@@ -47,7 +47,7 @@
 ##'                        betm0 = betm0, betQ0 = betQ0,
 ##'                        ssqdf = ssqdf, ssqsc = ssqsc,
 ##'                        phistart = parlist$phi[i], omgstart = parlist$omg[i],
-##'                        linkp = parlist$linkp[i], kappa = parlist$kappa[i], 
+##'                        linkp = parlist$linkp[i], kappa = parlist$kappa[i],
 ##'                        corrfcn = corrf, phisc = 0, omgsc = 0)
 ##' }
 ##' bf <- bf1skel(runs)
@@ -57,30 +57,33 @@
 ##' @importFrom sp spDists
 ##' @references Doss, H. (2010). Estimation of large families of Bayes
 ##' factors from Markov chain output. \emph{Statistica Sinica}, 20(2),
-##' 537. 
+##' 537.
 ##'
 ##' Roy, V., Evangelou, E., and Zhu, Z. (2015). Efficient estimation
 ##' and prediction for the Bayesian spatial generalized linear mixed
-##' model with flexible link functions. \emph{Biometrics}.
-##' \url{http://dx.doi.org/10.1111/biom.12371}
-##' @export 
-bf2new <- function (bf1obj, linkp, phi, omg, kappa, useCV = TRUE) {
-
+##' model with flexible link functions. \emph{Biometrics}, 72(1),
+##'   289-298.
+##' @useDynLib geoBayes calcb_no_st calcb_mu_st calcb_wo_st calcb_tr_st
+##' @useDynLib geoBayes calcb_no_cv calcb_mu_cv calcb_wo_cv calcb_tr_cv
+##' @export
+bf2new <- function (bf1obj, linkp, phi, omg, kappa, useCV = TRUE)
+{
   ## Logical input
   useCV <- as.logical(useCV)
 
   ## Extract model variables
   transf <- bf1obj$transf
+  real_transf <- bf1obj$real_transf
+  itr <- bf1obj$itr
   y <- bf1obj$response
   n <- length(y)
   l <- bf1obj$weights
   F <- bf1obj$modelmatrix
   p <- NCOL(F)
   family <- bf1obj$family
-  ifam <- match(family, eval(formals(mcsglmm)$family), 0L)
+  ifam <- .geoBayes_family(family)
   corrfcn <- bf1obj$corrfcn
-  needkappa <- corrfcn %in% c("matern", "powerexponential")
-  icf <- match(corrfcn, c("matern", "spherical", "powerexponential"))
+  icf <- .geoBayes_correlation(corrfcn)
   betm0 <- bf1obj$betm0
   betQ0 <- bf1obj$betQ0
   ssqdf <- bf1obj$ssqdf
@@ -99,37 +102,9 @@ bf2new <- function (bf1obj, linkp, phi, omg, kappa, useCV = TRUE) {
 
   ## Examine link function parameter
   if (missing(linkp) || is.null(linkp)) {
-    linkp <- nu <- unique(bf1obj$pnts$nu)
-  } else if (family == "binomial") {
-    if (is.character(linkp) | is.factor(linkp)) {
-      if (length(linkp) != 1) {
-        stop ("When using character linkp, it must not be a vector")
-      }
-      if (linkp == "logit") {
-        nu <- unique(bf1obj$pnts$nu)
-        if (!isTRUE(nu == -1)) {
-          stop ("The logit link is not consistent with the model in bf1obj")
-        }
-      } else if (linkp == "probit") {
-        nu <- unique(bf1obj$pnts$nu)
-        if (!isTRUE(nu == 0)) {
-          stop ("The logit link is not consistent with the model in bf1obj")
-        }
-      } else {
-        stop ("Unrecognised linkp")
-      }
-    } else if (is.numeric(linkp)) {
-      nu <- as.double(linkp)
-      if (any(nu <= 0)) {
-        stop ("Link parameter must be postive for the robit link")
-      }
-    } else {
-      stop ("Unrecognised linkp")
-    }
-  } else if (is.numeric(linkp)) {
-    nu <- as.double(linkp)
+    nu <- unique(bf1obj$pnts$nu)
   } else {
-    stop ("Unrecognised linkp")
+    nu <- unique(.geoBayes_getlinkp(linkp, ifam))
   }
   n_nu <- length(nu)
 
@@ -150,17 +125,10 @@ bf2new <- function (bf1obj, linkp, phi, omg, kappa, useCV = TRUE) {
   omg <- as.double(omg)
   if (any(omg < 0)) stop ("Argument omg must be non-negative")
   n_omg <- length(omg)
-  if (!needkappa || missing(kappa) || is.null(kappa)) {
+  if (missing(kappa) || is.null(kappa)) {
     kappa <- unique(bf1obj$pnts$kappa)
-  } else if (!is.numeric(kappa)) {
-    stop ("Argument kappa must be numeric or NULL")
-  }
-  kappa <- as.double(kappa)
-  if (corrfcn %in% c("matern", "powerexponential") && any(kappa < 0)) {
-    stop ("Argument kappa cannot be negative")
-  }
-  if (corrfcn == "powerexponential" && any(kappa > 2)) {
-    stop ("Argument kappa cannot be more than 2")
+  } else {
+    kappa <- unique(.geoBayes_getkappa(kappa, icf))
   }
   n_kappa <- length(kappa)
   covpars <- expand.grid(phi = phi, omg = omg, kappa = kappa)
@@ -175,43 +143,46 @@ overflow. Control variates corrections will not be used.")
     useCV <- FALSE
   }
 
-  if (transf) {
-    froutine <- "calcbmu"
-  } else if ((family == "binomial") && bf1obj$binwo) {
-    froutine <- "calcbw"
-  } else {
-    froutine <- "calcbz"
+  froutine <- paste0("calcb_", real_transf)
+  if (real_transf == "wo" || real_transf == "tr") {
+    ifam <- -ifam
   }
 
   if (useCV) {
     froutine <- paste(froutine, '_cv', sep='')
+    QRin <- QRintercept(zcv)
     RUN <- .Fortran(froutine, bfact,
                     as.double(covpars$phi), as.double(nu),
                     as.double(covpars$omg),
-                    as.double(covpars$kappa), as.integer(icf), 
+                    as.double(covpars$kappa), as.integer(icf),
                     as.integer(n_cov), as.integer(n_nu), as.integer(Ntot),
-                    as.double(sample), as.double(isweights), as.double(zcv),
-                    as.integer(n), as.integer(p), as.integer(kg),
+                    as.double(sample), as.double(isweights),
+                    as.double(QRin), as.integer(n), as.integer(p),
                     as.double(betm0),
                     as.double(betQ0), as.double(ssqdf), as.double(ssqsc),
                     max(tsqdf, 0), as.double(tsq), as.double(y),
-                    as.double(l), as.double(F), as.double(dm), as.integer(ifam))
+                    as.double(l), as.double(F), as.double(dm),
+                    as.integer(ifam), as.integer(itr),
+                    PACKAGE = "geoBayes")
   } else {
     froutine <- paste(froutine, '_st', sep='')
     RUN <- .Fortran(froutine, bfact,
                     as.double(covpars$phi), as.double(nu),
                     as.double(covpars$omg),
-                    as.double(covpars$kappa), as.integer(icf), 
+                    as.double(covpars$kappa), as.integer(icf),
                     n_cov, n_nu, Ntot, as.double(sample), as.double(isweights),
                     as.integer(n), as.integer(p), as.double(betm0),
                     as.double(betQ0), as.double(ssqdf), as.double(ssqsc),
                     max(tsqdf, 0), as.double(tsq), as.double(y), as.double(l),
-                    as.double(F), as.double(dm), as.integer(ifam))
+                    as.double(F), as.double(dm), as.integer(ifam),
+                    as.integer(itr),
+                    PACKAGE = "geoBayes")
   }
+
   logbf <- array(RUN[[1]], c(n_nu, n_phi, n_omg, n_kappa))
   logbf <- logbf + bf1obj$logbf[1] # Constant to match bf1obj$logbf
   maxid <- arrayInd(which.max(logbf), c(n_nu, n_phi, n_omg, n_kappa))
-  out <- list(logbf = logbf, linkp = linkp, phi = phi,
+  out <- list(logbf = logbf, linkp = nu, phi = phi,
               omg = omg, corrfcn = corrfcn, kappa = kappa, indmax = maxid)
   class(out) <- c("bfsp", "list")
   out
@@ -220,7 +191,7 @@ overflow. Control variates corrections will not be used.")
 
 ##' This function plots the estimated logarithm Bayes factors from the
 ##' function \code{\link{bf2new}}.
-##' 
+##'
 ##' Depending on whether \code{pars} has length 1 or 2, this function
 ##' creates a line or a contour plot of the estimated Bayes factors.
 ##' If its length is 3 or 4, then it produces multiple profile plots.
@@ -229,12 +200,12 @@ overflow. Control variates corrections will not be used.")
 ##' against that value.
 ##' @title Plot the estimated Bayes factors
 ##' @param bf2obj Output from the function \code{\link{bf2new}}.
-##' @param pars A vector with the names of the parameters to plot. 
+##' @param pars A vector with the names of the parameters to plot.
 ##' @param profile Whether it should produce a profile plot or a
 ##' contour plot if the length of pars is 2.
 ##' @param ... Other input to be passed to either \code{plot} or
 ##' \code{contour}.
-##' @return This function returns nothing. 
+##' @return This function returns nothing.
 ##' @examples \dontrun{
 ##' data(rhizoctonia)
 ##' ### Define the model
@@ -262,7 +233,7 @@ overflow. Control variates corrections will not be used.")
 ##'                        betm0 = betm0, betQ0 = betQ0,
 ##'                        ssqdf = ssqdf, ssqsc = ssqsc,
 ##'                        phistart = parlist$phi[i], omgstart = parlist$omg[i],
-##'                        linkp = parlist$linkp[i], kappa = parlist$kappa[i], 
+##'                        linkp = parlist$linkp[i], kappa = parlist$kappa[i],
 ##'                        corrfcn = corrf, phisc = 0, omgsc = 0)
 ##' }
 ##' bf <- bf1skel(runs)
@@ -271,12 +242,14 @@ overflow. Control variates corrections will not be used.")
 ##' plotbf2(bfall, c("phi", "omg"), profile = TRUE, type = "b", ylab="log(BF)")
 ##' }
 ##' @importFrom graphics plot points par title
-##' @export 
+##' @export
 plotbf2 <- function (bf2obj, pars = c("linkp", "phi", "omg", "kappa"),
                      profile = length(pars) > 2, ...) {
   N <- 4
   pars <- unique(pars)
   pars <- match.arg(pars, several.ok = TRUE)
+  usepars <- sapply(bf2obj[pars], length) > 1
+  pars <- pars[usepars]
   ipar <- match(pars, eval(formals()$pars))
   npars <- length(pars)
   if (npars == 0) stop ("No parameters to plot.")
@@ -297,7 +270,7 @@ plotbf2 <- function (bf2obj, pars = c("linkp", "phi", "omg", "kappa"),
 
   grknm <- list(nu = expression(nu), phi = expression(phi),
                 omg = expression(omega), kappa = expression(kappa))
-  
+
   if (ptype == "line") {
     maxid <- bf2obj$indmax
     ii <- as.list(maxid)
@@ -325,8 +298,10 @@ plotbf2 <- function (bf2obj, pars = c("linkp", "phi", "omg", "kappa"),
            (bf2obj[[pars[2]]])[maxid[ipar[2]]])
   } else if (ptype == "profile") {
     ii <- as.list(rep(TRUE, N))
-    oldpar <- par(mfrow = c(1, npars))
-    on.exit(par(oldpar), add = TRUE)
+    if (prod(par("mfrow")) < npars) {
+      oldpar <- par(mfrow = c(1, npars))
+      on.exit(par(oldpar), add = TRUE)
+    }
     for (i in 1:npars) {
       bf <- apply(bf2obj[["logbf"]], ipar[i], max)
       pdata <- data.frame(bf2obj[pars[i]], logbf = bf)
@@ -370,6 +345,8 @@ plotbf2 <- function (bf2obj, pars = c("linkp", "phi", "omg", "kappa"),
 ##' @param control A list of control parameters for the optimisation.
 ##' See \code{\link[stats]{optim}}.
 ##' @return The output from the function \code{\link[stats]{optim}}.
+##'   The \code{"value"} element is the log-Bayes factor, not the
+##'   negative log-Bayes factor.
 ##' @examples \dontrun{
 ##' data(rhizoctonia)
 ##' ### Define the model
@@ -397,7 +374,7 @@ plotbf2 <- function (bf2obj, pars = c("linkp", "phi", "omg", "kappa"),
 ##'                        betm0 = betm0, betQ0 = betQ0,
 ##'                        ssqdf = ssqdf, ssqsc = ssqsc,
 ##'                        phistart = parlist$phi[i], omgstart = parlist$omg[i],
-##'                        linkp = parlist$linkp[i], kappa = parlist$kappa[i], 
+##'                        linkp = parlist$linkp[i], kappa = parlist$kappa[i],
 ##'                        corrfcn = corrf, phisc = 0, omgsc = 0)
 ##' }
 ##' bf <- bf1skel(runs)
@@ -405,7 +382,10 @@ plotbf2 <- function (bf2obj, pars = c("linkp", "phi", "omg", "kappa"),
 ##' est
 ##' }
 ##' @importFrom stats optim
-##' @export 
+##' @useDynLib geoBayes calcb_no_st calcb_mu_st calcb_wo_st calcb_tr_st
+##' @useDynLib geoBayes calcb_no_cv calcb_mu_cv calcb_wo_cv calcb_tr_cv
+##' @useDynLib geoBayes calcbd_no calcbd_mu calcbd_wo calcbd_tr
+##' @export
 bf2optim <- function (bf1obj, paroptim, useCV = TRUE,
                       control = list()) {
 
@@ -414,16 +394,17 @@ bf2optim <- function (bf1obj, paroptim, useCV = TRUE,
 
   ## Extract model variables
   transf <- bf1obj$transf
+  real_transf <- bf1obj$real_transf
+  itr <- bf1obj$itr
   y <- bf1obj$response
   n <- length(y)
   l <- bf1obj$weights
   F <- bf1obj$modelmatrix
   p <- NCOL(F)
   family <- bf1obj$family
-  ifam <- match(family, eval(formals(mcsglmm)$family), 0L)
+  ifam <- .geoBayes_family(family)
   corrfcn <- bf1obj$corrfcn
-  needkappa <- corrfcn %in% c("matern", "powerexponential")
-  icf <- match(corrfcn, c("matern", "spherical", "powerexponential"))
+  icf <- .geoBayes_correlation(corrfcn)
   betm0 <- bf1obj$betm0
   betQ0 <- bf1obj$betQ0
   ssqdf <- bf1obj$ssqdf
@@ -438,113 +419,16 @@ bf2optim <- function (bf1obj, paroptim, useCV = TRUE,
   Ntot <- NCOL(sample)
   isweights <- bf1obj$isweights
   zcv <- bf1obj$controlvar
-  nruns <- NCOL(zcv)
-  linkp <- if (family == "binomial") {
-    if (bf1obj$pnts$nu[1] < 0) {
-      "logit"
-    } else if (bf1obj$pnts$nu[1] == 0) {
-      "probit"
-    } else "robit"
-  } else "boxcox"
-  parnmall <- c("linkp", "phi", "omg", "kappa")
-  parnm <- c("linkp", "phi", "omg", if (needkappa) "kappa")
-  
-  ## Read and check paroptim argument
-  if (!is.list(paroptim)) {
-    stop ("Argument paroptim must be a list")
-  }
-  if (!all(parnm %in% names(paroptim))) {
-    stop (paste("Named components in the argument paroptim",
-                "must be", paste(parnm, collapse = " ")))
-  } else {
-    paroptim <- paroptim[parnm]
-  }
-  if (!needkappa) paroptim$kappa <- 0
-  lower <- upper <- pstart <- rep.int(NA, 4)
-  estim <- rep.int(FALSE, 4)
-  linkpe <- paroptim[["linkp"]]
-  if (is.character(linkpe) | is.factor(linkpe)) {
-    if (family == "binomial") {
-      if (linkpe == "logit") {
-        if (linkp == "logit") {
-          pstart[1] <- -1
-          estim[1] <- FALSE
-        } else {
-          stop ("Link in paroptim inconsistent with link in bf1obj")
-        }
-      } else if (linkpe == "probit") {
-        if (linkp == "probit") {
-          pstart[1] <- 0
-          estim[1] <- FALSE
-        } else {
-          stop ("Link in paroptim inconsistent with link in bf1obj")
-        }
-      } else {
-        stop ("Character link for the binomial family can be either
-\"logit\" or \"probit\" in argument paroptim")
-      }
-    } else stop ("Character link in argument paroptim is only used for
-the binomial family")
-  } else if (is.numeric(linkpe)) {
-    if (length(linkpe) < 1 | length(linkpe) > 3) {
-      stop ("The length for a numeric component in paroptim must be 1, 2, or 3")
-    } else if (length(linkpe) == 1) {
-      pstart[1] <- linkpe
-      estim[1] <- FALSE
-    } else if (length(linkpe) == 2) {
-      pstart[1] <- NA
-      estim[1] <- TRUE
-      lower[1] <- linkpe[1]
-      upper[1] <- linkpe[2]
-      if (lower[1] >= upper[1]) {
-        stop ("The lower bound must be less than the upper bound for linkp
-in paroptim")
-      }
-    } else {
-      pstart[1] <- linkpe[2]
-      estim[1] <- TRUE
-      lower[1] <- linkpe[1]
-      upper[1] <- linkpe[3]
-      if (lower[1] > pstart[1] | pstart[1] > upper[1] | lower[1] == upper[1]) {
-        stop ("The elements in the component linkp in paroptim must be ordered")
-      }
-    }
-  } else {
-    stop ("The element linkp in paroptim must be either numeric or character")
-  }
-  for (i in 2:4) {
-    ppp <- paroptim[[parnmall[i]]]
-    if (!is.numeric(ppp)) {
-      stop(paste("The element", parnmall[i], "in paroptim must be numeric"))
-    }
-    lppp <- length(ppp)
-    if (lppp < 1 | lppp > 3) {
-      stop (paste("The element", parnmall[i], "in paroptim must have 1, 2, or 3
-components"))
-    }
-    if (lppp == 1) {
-      pstart[i] <- ppp
-      estim[i] <- FALSE
-    } else if (lppp == 2) {
-      pstart[i] <- NA
-      estim[i] <- TRUE
-      lower[i] <- ppp[1]
-      upper[i] <- ppp[2]
-      if (lower[i] >= upper[i]) {
-        stop (paste("The lower bound must be less than the upper bound for",
-                     parnmall[i], "in paroptim"))
-      }
-    } else {
-      pstart[i] <- ppp[2]
-      estim[i] <- TRUE
-      lower[i] <- ppp[1]
-      upper[i] <- ppp[3]
-      if (lower[i] > pstart[i] | pstart[i] > upper[i] | lower[i] == upper[i]) {
-        stop (paste("The elements in the component", parnmall[i],
-                     "in paroptim must be ordered"))
-      }
-    }
-  }
+  kg <- NCOL(zcv)
+
+
+  ## Check paroptim
+  ## parnmall <- c("linkp", "phi", "omg", "kappa")
+  paroptim <- getparoptim(paroptim, ifam, icf)
+  pstart <- as.double(paroptim$pstart)
+  lower <- paroptim$lower
+  upper <- paroptim$upper
+  estim <- paroptim$estim
 
   ## Check for non-finite values in control variates
   if (useCV && any(!is.finite(zcv))) {
@@ -553,53 +437,78 @@ overflow. Control variates corrections will not be used.")
     useCV <- FALSE
   }
 
+  froutine <- paste0("calcb_", real_transf)
+  groutine <- paste0("calcbd_", real_transf)
+  if (real_transf == "wo" || real_transf == "tr") {
+    ifam <- -ifam
+  }
+
   ## Function to optimise
   if (useCV) {
-    if (transf) {
-      froutine <- "calcbmu_cv"
-    } else if ((family == "binomial") && bf1obj$binwo) {
-      froutine <- "calcbw_cv"
-    } else {
-      froutine <- "calcbz_cv"
-    }
+    froutine <- paste(froutine, "_cv", sep = "")
+    QRin <- QRintercept(zcv)
     fn <- function (par) {
-      parin <- as.double(pstart)
+      parin <- pstart
       parin[estim] <- par
-      RUN <- .Fortran(froutine, 0.0, parin[2], parin[1], parin[3], parin[4],
+      parin[2:4] <- exp(parin[2:4])
+      RUN <- .Fortran(froutine, 0,
+                      parin[2], parin[1], parin[3], parin[4],
                       as.integer(icf), 1L, 1L, as.integer(Ntot),
-                      as.double(sample), as.double(isweights), as.double(zcv),
+                      as.double(sample), as.double(isweights),
+                      as.double(QRin),
                       as.integer(n), as.integer(p),
-                      as.integer(nruns), as.double(betm0),
+                      as.double(betm0),
                       as.double(betQ0), as.double(ssqdf), as.double(ssqsc),
                       max(tsqdf, 0), as.double(tsq), as.double(y),
                       as.double(l), as.double(F), as.double(dm),
-                      as.integer(ifam))
+                      as.integer(ifam), as.integer(itr),
+                      PACKAGE = "geoBayes")
       -RUN[[1]][1]
     }
   } else {
-    if (transf) {
-      froutine <- "calcbmu_st"
-    } else if ((family == "binomial") && bf1obj$binwo) {
-      froutine <- "calcbw_st"
-    } else {
-      froutine <- "calcbz_st"
-    }
+    QRin <- numeric(Ntot)
+    froutine <- paste(froutine, "_st", sep = "")
     fn <- function (par) {
-      parin <- as.double(pstart)
+      parin <- pstart
       parin[estim] <- par
-      RUN <- .Fortran(froutine, 0.0, parin[2], parin[1], parin[3], parin[4],
-                      as.integer(icf), 1L, 1L, as.integer(Ntot), as.double(sample),
+      parin[2:4] <- exp(parin[2:4])
+      RUN <- .Fortran(froutine, 0,
+                      parin[2], parin[1], parin[3], parin[4],
+                      as.integer(icf), 1L, 1L,
+                      as.integer(Ntot), as.double(sample),
                       as.double(isweights), as.integer(n), as.integer(p),
                       as.double(betm0),
                       as.double(betQ0), as.double(ssqdf), as.double(ssqsc),
                       max(tsqdf, 0), as.double(tsq), as.double(y), as.double(l),
                       as.double(F), as.double(dm),
-                      as.integer(ifam))
+                      as.integer(ifam), as.integer(itr),
+                      PACKAGE = "geoBayes")
       -RUN[[1]][1]
     }
   }
-  
-  method <- if (sum(estim) == 1) "Brent" else "L-BFGS-B"
+
+### Compute derivative
+  gr <- function (par) {
+    parin <- pstart
+    parin[estim] <- par
+    parin[2:4] <- exp(parin[2:4])
+    RUN <- .Fortran(groutine, 0, 0, 0, 0, 0,
+                    parin[2], parin[1], parin[3], parin[4],
+                    as.integer(icf), as.integer(Ntot), as.double(sample),
+                    as.double(isweights),
+                    as.double(QRin),
+                    as.integer(n), as.integer(p),
+                    as.double(betm0),
+                    as.double(betQ0), as.double(ssqdf), as.double(ssqsc),
+                    max(tsqdf, 0), as.double(tsq), as.double(y), as.double(l),
+                    as.double(F), as.double(dm),
+                    as.integer(ifam), as.integer(itr), as.integer(useCV),
+                    PACKAGE = "geoBayes")
+    gg <- -do.call("c", RUN[2:5])*c(1, parin[2:4])
+    gg[estim]
+  }
+
+
   logbf <- bf1obj$logbf
   imaxlogbf <- which.max(logbf)
   wbf <- exp(logbf - logbf[imaxlogbf] - log(sum(exp(logbf - logbf[imaxlogbf]))))
@@ -607,13 +516,47 @@ overflow. Control variates corrections will not be used.")
                       wbf)
   i <- is.na(pstart) & estim
   pstart[i] <- pmax(pmin(upper[i], pstart.d[i]), lower[i])
-  op <- stats::optim(pstart[estim], fn, method = method,
+  scale <- control$parscale; if (is.null(scale)) scale <- 1
+  pstart[2:4] <- log(pstart[2:4])
+  lower[2:4] <- log(lower[2:4])
+  upper[2:4] <- log(upper[2:4])
+  method <- "L-BFGS-B"
+  if (isTRUE(control$fnscale < 0)) control$fnscale <- -control$fnscale
+###   op <- stats::nlminb(pstart[estim], fn, gr, scale = scale,
+###                       control = control,
+###                       lower = lower[estim], upper = upper[estim])
+###   op$value <- -op$objective + bf1obj$logbf[1]
+###   op$hessian <- optimHess(op$par, fn, gr, control = control)
+###   op <- optimx::optimx(pstart[estim], fn, gr, method = method,
+###                        lower = lower[estim], upper = upper[estim],
+###                        control = control, hessian = TRUE)
+  op <- stats::optim(pstart[estim], fn, gr, method = method,
                      lower = lower[estim], upper = upper[estim],
-                     control = control)
+                     control = control, hessian = TRUE)
+  op$value <- -op$value + bf1obj$logbf[1]
   parout <- pstart
   parout[estim] <- op$par
   names(parout) <- c("linkp", "phi", "omg", "kappa")
-  op$par <- parout
-  op$value <- -op$value + bf1obj$logbf[1]
+  op$par <- c(parout[1], exp(parout[2:4]))
+  op$hessian <- op$hessian*tcrossprod(c(1, op$par[2:4])[estim])
+  op$fixed <- !estim
   op
 }
+
+
+QRintercept <- function(zcv) {
+  ## Retrun the column from Q-R that corresponds to the intercept.
+  QR <- qr(zcv)
+  Rcv <- qr.R(QR)
+  Qcv <- qr.Q(QR)
+###   tol <- max(abs(diag(Rcv))) * NROW(zcv) * .Machine$double.eps
+###   ii <- abs(diag(Rcv)) > tol
+###   zcvrnk <- sum(ii)
+  zcvii <- QR$pivot == 1
+##   zcvQR <- Qcv
+##   zcvQR <- .Fortran("dtrsm", "r", "u", "t", "n", as.integer(Ntot),
+##                     as.integer(zcvrnk), 1.0, as.double(Rcv),
+##                     as.integer(kg), as.double(zcvQR), as.integer(Ntot))[[10]]
+  ## zcvQR <- t(backsolve(Rcv,t(Qcv)))
+  backsolve(Rcv,t(Qcv))[zcvii, ]
+}
